@@ -16,6 +16,7 @@
 #' concatenated
 #' @param dif logical: calculate difference if two fields containing numbers
 #' are selected.
+#' @param as.factor logical: if TRUE, a factor vector will be returned, strings otherwise.
 #' @return returns a vector containing factors or numbers
 #' @author Stefan Schlager
 #' 
@@ -34,66 +35,47 @@
 #' 
 #' 
 #' @export
-name2factor <- function(x,sep="_",which,collapse=sep)
-  {
-    if (length(dim(x))==3)
-      {
+name2factor <- function(x,sep="_",which,collapse=sep, as.factor=TRUE) {
+    if (length(dim(x))==3) {
         names <- dimnames(x)[[3]]
-      }
-    else if (length(dim(x))==2)
-      {
-        names <- dimnames(x)[[1]]
-      }
-    else if (is.list(x))
-      {        
-        names <- names(x)
-      }
-    else
-      {
-        names <- x
-      }
+    } else if (length(dim(x))==2) {
+          names <- dimnames(x)[[1]]
+      } else if (is.list(x)) {        
+            names <- names(x)
+        } else {
+              names <- x
+          }
     fac <- strsplit(names,split=sep)
-    
-    for (i in 1:length(fac))
-      {                 
+    for (i in 1:length(fac)) {                 
         fac[[i]] <- paste(fac[[i]][which],collapse=collapse)
-      }
-    fac <- as.factor(unlist(fac))
+    }
+    
+    fac <- unlist(fac)
+    if (as.factor)
+        fac <- factor(fac)
     return(fac)
-  }
+}
 #' @rdname name2factor
 #' @export
-name2num <- function(x,sep="_",which,collapse=sep,dif=TRUE)
-  {
-    if (length(dim(x))==3)
-      {
+name2num <- function(x,sep="_",which,collapse=sep,dif=TRUE) {
+    if (length(dim(x))==3) {
         names <- dimnames(x)[[3]]
-      }
-    else if (length(dim(x))==2)
-      {
-        names <- dimnames(x)[[1]]
-      }
-    else if (is.list(x))
-      {        
-        names <- names(x)
-      }
-    else
-      {
-        names <- x
-      }
+    } else if (length(dim(x))==2) {
+          names <- dimnames(x)[[1]]
+      } else if (is.list(x)) {        
+            names <- names(x)
+        } else {
+              names <- x
+          }
     fac <- strsplit(names,split=sep)
     
-    for (i in 1:length(fac))
-      {
-        if (dif && length(which)>1)
-          {
+    for (i in 1:length(fac)) {
+        if (dif && length(which) > 1) {
             fac[[i]] <- diff(as.numeric(fac[[i]][which]))
+        } else {
+              fac[[i]] <- as.numeric(fac[[i]][which])
           }
-        else
-          {
-            fac[[i]] <- as.numeric(fac[[i]][which])
-          }
-      }
+    }
     fac <- (unlist(fac))
     return(fac)
-  }
+}
